#!/usr/bin/perl -w
#
# Pug v0.1.1
# Copyright Tod D. Ihde - 14 September, 1998
#
# ( C News. C News run. C Pug. C Pug run. Run, Pug, run! C Pug catch News. )
#
# This script controls the execution of newsrun, sendbatches, and newsx
# for a C-News based system running newsx as it's transport agent.
# Really, all systems should be running newsx.
#
# This program has been released under the GNU General Public License
# as published by the Free Software Foundation; version 2 dated June, 1991.
# This means that I still hold the copyright.
# Please see the file LICENSE for details.
#
# Comments? Suggestions? Flames? Cookies?
# Email: toon@waste.org
#
use strict;
use Fcntl;

my ($mknod, $fifo, $fifomode, @message, @servers, $server, $name);
my ($sys, $newsrun, $sendbatches, $newsx, $newsxargs, $newsxlocks);

# Like the pine trees lining the winding road,
# I got a name, I got a name.
$0="Pug";
$name=$0;

# You can change these to suit your needs.
#
# Programs
$mknod="/bin/mknod";
$newsrun="/usr/lib/news/bin/input/newsrun";
$sendbatches="/usr/lib/news/bin/batch/sendbatches";
$newsx="/usr/local/bin/newsx";
#
# Files
$fifo="/var/lib/news/.Pug.fifo";
$sys="/var/lib/news/sys";
$newsxlocks="/var/lib/news/LOCKnewsx*";
#
# Misc.
$fifomode="770";
$newsxargs="--maxnew 200 --window 10 --nopost";
# That's it for configurability!

$0="$name - Initializing.";

# Subroutines:

# Remove any old FIFO and create a new one (nonblocking).
sub make_fifo ($nonblocking) {
	my ($nonblocking);
	$nonblocking=@_;
	unlink($fifo) if ( -e $fifo);
	die "Can't control $fifo" if ( -e $fifo );
	if(system("$mknod -m $fifomode $fifo p")) {
		die "Couldn't mknod $fifo";
		}

	if ($nonblocking) {
		# Open it nonblocking (We continue if there's no data).
		sysopen(FIFO,"$fifo", O_NONBLOCK) || die "WTF? Can't read from $fifo!";

		} else {
		# We want to wait for data (blocking read).
		open(FIFO,"< $fifo") || die "WTF? Can't read from $fifo!";
		}
	}

# Read in the list of news servers to get news from.
sub read_systems {
	my (%saw, @saw, @servers);
	open(SYS,"$sys") || die "WTF? Can't read sys file!";
	@servers=<SYS>;
	close(SYS);

	# Yummy!
	chomp(@servers);

	for (@servers) {
		# Strip comments, and all fields 'cept the address.
		s/([^:|#]*)#*.*$/$1/;
		# Strip out myself.
		s/ME.*//;
		# Strip spaces.
		s/ //;
		# Get rid of blank lines.
		if ($_) { push @saw,$_ };
		}
	@servers=@saw;

	# Sort the list... Don't even really need this... I think...
	undef %saw;
	@saw{@servers}=();
	@servers=sort keys %saw;
	undef %saw;

	return (@servers);
	}

sub check_fifo() {
	my ($message);
	$message=<FIFO>;
	return if (!$message);
	die "Recieved DIE command" if ($message=~m/[d|D][i|I][e|E]/);
	return if (!$message=~m/[s|S][l|L][e|E][e|E][p|P]/);
	$0="$name - sleeping...";
	while(1) {
		close(FIFO);
		&make_fifo();
		$message=<FIFO>;
		die "Recieved DIE command" if ($message=~m/[d|D][i|I][e|E]/);
		close(FIFO) if ($message=~m/[c|C][o|O][n|N][t|T]/);
		&make_fifo(1) if ($message=~m/[c|C][o|O][n|N][t|T]/);
		last if ($message=~m/[c|C][o|O][n|N][t|T]/);
		}
	}
		
# Main program:
while (1) {
	&make_fifo(1);
	@servers=&read_systems;
	for (@servers) {
		$server=$_;
		# Remove any left-over lockfiles from god-knows-what.
		# This should really be taken care of by something else...
		# Comment it out if it scares you. ;)
		#unlink ("/var/lib/news/L\.*");

		# Newsrun - Unbatches all recieved news & updates the active
		# file, etc...
		$0="$name - newsrun";
		system("$newsrun");
		&check_fifo;

		# Sendbatches - Gets outgoing batches ready to send.
		$0="$name - sendbatches";
		system("$sendbatches");
		&check_fifo;

		# Silly crackers are always leaving their laundry detergent
		# on the playground...
		unlink ("$newsxlocks");
		# NewsX - Sends & recieves new news.
		$0="$name - newsx";
		system("$newsx $newsxargs $server");
		&check_fifo;
		}
	}
